#include "Code6.h"

CMyWinApp MyApplication;

BOOL CMyWinApp::InitInstance()
{
    CCode6* pFrame = new CCode6;
    m_pMainWnd = pFrame;
    pFrame->ShowWindow(SW_SHOW);
    pFrame->UpdateWindow();
    return TRUE;
}

BEGIN_MESSAGE_MAP(CCode6,CFrameWnd)
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
    ON_BN_CLICKED(IDC_BUTTON,OnButton)
END_MESSAGE_MAP()

CCode6::CCode6()
{
	Create(NULL,"Code6: Nonlinear Equations",
		WS_OVERLAPPEDWINDOW,CRect(0,0,800,635),NULL);
	Arial80.CreatePointFont(80,"Arial");
	pt=new PT [m+1];
	fMenu=0; fStatus=0; idc=301;
	menu[1].item="NLE: Bisection"; 
	menu[2].item="NLE: False-Point Position";
	menu[3].item="NLE: Newton-Raphson";
	menu[4].item="NLE: Secant";
	for (int i=1;i<=nMenuItems;i++)
	{
		menu[i].hm=CPoint(20,30+(i-1)*30);
		menu[i].rc=CRect(menu[i].hm.x,menu[i].hm.y,
			menu[i].hm.x+150,menu[i].hm.y+20);
	}
	curve.hm=CPoint(320,320); curve.end=CPoint(760,590);
	curve.rc=CRect(curve.hm.x-10,curve.hm.y-10,
		curve.end.x+10,curve.end.y+10);
	input[0].hm=CPoint(200,10);
	input[0].rc=CRect(input[0].hm.x,input[0].hm.y,
		input[0].hm.x+560,input[0].hm.y+260);
	input[0].display=CRect(30,180,170,250);
	for (i=1;i<=maxInput;i++)
		input[i].hm=CPoint(input[0].hm.x+10,input[0].hm.y+30+(i-1)*25);
}

CCode6::~CCode6()
{	
	delete pt;
}

void CCode6::OnPaint()
{
	CPaintDC dc(this);
	CString str;
	dc.SelectObject(Arial80);
	dc.SetBkColor(RGB(150,150,150));
	dc.SetTextColor(RGB(255,255,255));
	for (int i=1;i<=nMenuItems;i++)
	{
		dc.FillSolidRect(&menu[i].rc,RGB(150,150,150));
		dc.TextOut(menu[i].hm.x+5,menu[i].hm.y+5,menu[i].item);
	}
	dc.SetBkColor(RGB(255,255,255));
	dc.SetTextColor(RGB(100,100,100));
	dc.Rectangle(input[0].rc);
	dc.TextOut(input[0].hm.x+150,
		input[0].hm.y+5,menu[fMenu].item);
	if (fMenu!=0)
	{
		for (i=1;i<=nInputItems;i++)
			dc.TextOut(input[i].hm.x+10,
			input[i].hm.y,input[i].label);
		if (fStatus)
		{
			DrawCurve();
			str.Format("Solution: x=%lf",Solution);
			dc.TextOut(input[0].display.left,input[0].display.top,str);
			fStatus=0;
		}
		else
			dc.TextOut(input[0].display.left,input[0].display.top,
				"No solution");
	}
}

void CCode6::DrawCurve()
{
	CClientDC dc(this);
	int i,psi[2];
	double h,m1,m2,c1,c2,psv[2];
	CString str;
	CPoint px;
	psi[1]=23;
	if (fMenu==1 || fMenu==2)
	{
		left.x=pt[0].a;
		right.x=pt[0].b;
	}
	if (fMenu==3 || fMenu==4)
	{
		left.x=Solution-2;
		right.x=Solution+2;
	}
	h=(right.x-left.x)/(double)m;
	pt[0].x=left.x;
	psv[1]=pt[0].x; 
	pt[0].y=parse(input[1].item,1,psv,psi);
	max.y=pt[0].y; min.y=pt[0].y;
	for (i=1;i<=m;i++)
	{
		pt[i].x=pt[i-1].x+h;
		psv[1]=pt[i].x; 
		pt[i].y=parse(input[1].item,1,psv,psi);
		if (max.y<pt[i].y)
			max.y=pt[i].y;	
		if (min.y>pt[i].y)
			min.y=pt[i].y;	
	}

	// Cartesian-Windows conversion coordinates
	m1=(double)(curve.end.x-curve.hm.x)/(right.x-left.x);
	c1=(double)curve.hm.x-left.x*m1;
	m2=(double)(curve.hm.y-curve.end.y)/(max.y-min.y);
	c2=(double)curve.end.y-min.y*m2;

	// Draw & label the x,y axis
	CPen pGray(PS_SOLID,1,RGB(100,100,100));
	dc.SelectObject(pGray);
	dc.SelectObject(Arial80);
	px=CPoint(m1*0+c1,m2*min.y+c2); dc.MoveTo(px);
	px=CPoint(m1*0+c1,m2*max.y+c2); dc.LineTo(px);
	
	px=CPoint(m1*left.x+c1,m2*0+c2); dc.MoveTo(px);
	str.Format("%.0lf",left.x); dc.TextOut(px.x,px.y,str);
	px=CPoint(m1*right.x+c1,m2*0+c2); dc.LineTo(px);
	str.Format("%.0lf",right.x); dc.TextOut(px.x-10,px.y,str);

	// draw the curve
	CPen pDark(PS_SOLID,2,RGB(50,50,50));
	dc.SelectObject(pDark);
	for (i=0;i<=m;i++)
	{
		px=CPoint((int)(m1*pt[i].x+c1),(int)(m2*pt[i].y+c2));
		if (i==0)
			dc.MoveTo(px);
		else
			dc.LineTo(px);
		if (pt[i].y==max.y)
		{
			str.Format("%.6lf",max.y); 
			dc.TextOut(px.x,px.y-10,str);
		}
		if (pt[i].y==min.y)
		{
			str.Format("%.6lf",min.y); 
			dc.TextOut(px.x,px.y,str);
		}
	}
}

void CCode6::OnLButtonDown(UINT nFlags,CPoint pt)
{
	int i,k;
	for (k=1;k<=nMenuItems;k++)		// menu items
		if (menu[k].rc.PtInRect(pt))
		{
			fMenu=k;
			InvalidateRect(input[0].rc);
			switch(k)
			{
				case 1: 
				case 2: 
					nInputItems=4; 
					input[1].label="y=f(x)"; 
					input[2].label="a[0]"; 
					input[3].label="b[0]";
					input[4].label="epsilon";
					break;
				case 3: 
					nInputItems=5; 
					input[1].label="y=f(x)"; 
					input[2].label="y=f'(x)"; 
					input[3].label="x[0]"; 
					input[4].label="x[m]";
					input[5].label="epsilon"; 
					break;
				case 4: 
					nInputItems=5; 
					input[1].label="y=f(x)"; 
					input[2].label="x[0]"; 
					input[3].label="x[1]";
					input[4].label="x[m]";
					input[5].label="epsilon"; 
					break;
			}
			for (int i=1;i<=maxInput-1;i++)
				input[i].ed.DestroyWindow();
			btn.DestroyWindow();
			btn.Create("Compute",WS_CHILD| WS_VISIBLE| BS_DEFPUSHBUTTON,
				CRect(CPoint(input[0].hm.x+10,input[0].hm.y+5),
				CSize(100,20)),this,IDC_BUTTON);
			for (i=1;i<=nInputItems;i++)
				input[i].ed.Create(WS_CHILD| WS_VISIBLE| WS_BORDER, 
					CRect(input[i].hm.x+100,input[i].hm.y,
					input[i].hm.x+520,input[i].hm.y+20),this,idc++);
		}
}

void CCode6::OnButton()
{
	CString str;
	if (fMenu!=0)
	{
		for (int i=1;i<=nInputItems;i++)
			input[i].ed.GetWindowText(input[i].item);
		if (fMenu==1 || fMenu==2)
			BisectionFPP();
		if (fMenu==3)
			Newton();
		if (fMenu==4)
			Secant();
		if (fStatus)
		{
			ShowTable();
			InvalidateRect(curve.rc);
			InvalidateRect(input[0].display);
		}
		else
			InvalidateRect(input[0].display);
	}
}

void CCode6::BisectionFPP()
{
	int i,psi[2];
	double fa,fb,fc,psv[2],epsilon;
	pt[0].a=atof(input[2].item);
	pt[0].b=atof(input[3].item);
	epsilon=atof(input[4].item);
	psi[1]=23; psv[1]=pt[0].a; 
	fa=parse(input[1].item,1,psv,psi);
	psv[1]=pt[0].b; 
	fb=parse(input[1].item,1,psv,psi);
	if (fa*fb<0)
	{
		
		for (i=0;i<=maxIter;i++)
		{
			if (fMenu==1)
				pt[i].c=(pt[i].a+pt[i].b)/2;
			if (fMenu==2)
				pt[i].c=(pt[i].a*fb-pt[i].b*fa)/(fb-fa);
			psv[1]=pt[i].a; 
			fa=parse(input[1].item,1,psv,psi);
			psv[1]=pt[i].b; 
			fb=parse(input[1].item,1,psv,psi);
			psv[1]=pt[i].c; pt[i].x=pt[i].c;
			fc=parse(input[1].item,1,psv,psi); pt[i].y=fc;
			if (fa*fc>0)
			{
				pt[i+1].a=pt[i].c; pt[i+1].b=pt[i].b;
			}
			else
			{
				pt[i+1].b=pt[i].c; pt[i+1].a=pt[i].a;
			}
			if (i>0)
			{
				pt[i].error=fabs(pt[i].c-pt[i-1].c);
				if (pt[i].error<epsilon)
				{
					Stop=i; 
					Solution=pt[Stop].c;
					fStatus=1;
					break;
				}
			}
		}
	}
}

void CCode6::Newton()
{
	int i,psi[2];
	double epsilon,psv[2];
	pt[0].x=atof(input[3].item);
	pt[m].x=atof(input[4].item);
	epsilon=atof(input[5].item);
	psi[1]=23;
	for (i=0;i<=maxIter;i++)
	{
		psv[1]=pt[i].x; 
		pt[i].y=parse(input[1].item,1,psv,psi);
		pt[i].yd1=parse(input[2].item,1,psv,psi);
		if (i<m)
		{
			pt[i+1].x=pt[i].x-pt[i].y/pt[i].yd1;
			pt[i].error=fabs(pt[i+1].x-pt[i].x);
		}
		if (pt[i].error<epsilon)
		{
			Stop=i; 
			Solution=pt[Stop].x;
			fStatus=1; 
			break;
		}
	}
}

void CCode6::Secant()
{
	int i,psi[2];
	double epsilon,psv[2];
	pt[0].x=atof(input[2].item);
	pt[1].x=atof(input[3].item);
	pt[m].x=atof(input[4].item);
	epsilon=atof(input[5].item);
	psi[1]=23; psv[1]=pt[0].x; 
	pt[0].y=parse(input[1].item,1,psv,psi);
	for (i=0;i<=maxIter;i++)
	{
		if (i<m-2)
		{
			psv[1]=pt[i+1].x;
			pt[i+1].y=parse(input[1].item,1,psv,psi);
			pt[i+2].x=(pt[i].x*pt[i+1].y-pt[i+1].x*pt[i].y)/(pt[i+1].y-pt[i].y);
			pt[i].error=fabs(pt[i+2].x-pt[i+1].x);
		}
		if (pt[i].error<epsilon)
		{
			Stop=i; 
			Solution=pt[Stop].x;
			fStatus=1;
			break;
		}
	}
}

void CCode6::ShowTable()
{
	CString str;
	CPoint hTable=CPoint(20,310);
	CRect rcTable=CRect(hTable.x,hTable.y,hTable.x+280,hTable.y+290);
	table.DestroyWindow();
    table.Create(WS_VISIBLE | WS_CHILD | WS_DLGFRAME | LVS_REPORT
        | LVS_NOSORTHEADER,rcTable,this,idc++);
	table.InsertColumn(0,"i",LVCFMT_CENTER,25);
	table.InsertColumn(1,"x",LVCFMT_CENTER,70);
	table.InsertColumn(2,"f(x)",LVCFMT_CENTER,70);
	table.InsertColumn(3,"error",LVCFMT_CENTER,70);
	for (int i=0;i<=Stop;i++)
	{
		str.Format("%d",i); table.InsertItem(i,str,0);
		str.Format("%lf",pt[i].x); table.SetItemText(i,1,str);
		str.Format("%lf",pt[i].y); table.SetItemText(i,2,str);
		if (fMenu>2 || ((fMenu<=2) && i>0))
		{
			str.Format("%lf",pt[i].error); 
			table.SetItemText(i,3,str);
		}
	}
}
