#include "Code7.h"

CMyWinApp MyApplication;

BOOL CMyWinApp::InitInstance()
{
    CCode7* pFrame = new CCode7;
    m_pMainWnd = pFrame;
    pFrame->ShowWindow(SW_SHOW);
    pFrame->UpdateWindow();
    return TRUE;
}

BEGIN_MESSAGE_MAP(CCode7,CFrameWnd)
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
END_MESSAGE_MAP()

CCode7::CCode7()
{
	Create(NULL,"Code7: Curve fittings through interpolation/approximation",
		WS_OVERLAPPEDWINDOW,CRect(0,0,800,500),NULL);
	Arial80.CreatePointFont(80,"Arial");
	pt=new PT [m+1]; 
	ipt=new PT [maxPt+1]; 
	nPt=0; fMenu=0; idc=301; fStatus=0;
	menu[1].item="Lagrange";
	menu[2].item="Newton's Divided Difference";
	menu[3].item="Cubic Spline";
	menu[4].item="Least-Square (Linear)";
	menu[5].item="Least-Square (Cubic)";
	for (int i=1;i<=nMenuItems;i++)
	{
		menu[i].hm=CPoint(20,30+(i-1)*30);
		menu[i].rc=CRect(menu[i].hm.x,menu[i].hm.y,
			menu[i].hm.x+200,menu[i].hm.y+20);
	}
	curve.hm=CPoint(240,20); curve.end=CPoint(760,345);
	curve.rc=CRect(curve.hm.x-10,curve.hm.y-10,curve.end.x+10,curve.end.y+10);
	output.hm=CPoint(240,370); output.end=CPoint(760,450);
	output.rc=CRect(output.hm.x-10,output.hm.y-10,output.end.x+10,output.end.y+10);
	left.x=-5; right.x=5;
	min.y=-10; max.y=10;
	m1=(double)(curve.end.x-curve.hm.x)/(right.x-left.x);
	c1=(double)curve.hm.x-left.x*m1;
	m2=(double)(curve.hm.y-curve.end.y)/(max.y-min.y);
	c2=(double)curve.end.y-min.y*m2;
	A=new double [maxPt+1];
	B=new double [maxPt+1];
	C=new double [maxPt+1];
	D=new double [maxPt+1];
	c=new double [4];
}

CCode7::~CCode7()
{	
	delete pt,ipt,A,B,C,D,c;
}

void CCode7::OnPaint()
{
	int i,k;
	CPaintDC dc(this);
	CString str;
	CPoint px;
	CRect rc;
	dc.SetBkColor(RGB(150,150,150));
	dc.SetTextColor(RGB(255,255,255));
	dc.SelectObject(Arial80);
	for (i=1;i<=nMenuItems;i++)
	{
		dc.FillSolidRect(&menu[i].rc,RGB(150,150,150));
		dc.TextOut(menu[i].hm.x+5,menu[i].hm.y+5,menu[i].item);
	}
	dc.SetBkColor(RGB(255,255,255));
	dc.SetTextColor(RGB(100,100,100));
	dc.Rectangle(curve.rc);

	// Draw & label the x,y axes
	CPen pGray(PS_SOLID,1,RGB(100,100,100));
	dc.SelectObject(pGray);
	px=CPoint(m1*0+c1,m2*min.y+c2); dc.MoveTo(px);
	px=CPoint(m1*0+c1,m2*max.y+c2); dc.LineTo(px);	
	px=CPoint(m1*left.x+c1,m2*0+c2); dc.MoveTo(px);
	px=CPoint(m1*right.x+c1,m2*0+c2); dc.LineTo(px);
	dc.SelectObject(Arial80);
	str.Format("%.0lf",left.x); dc.TextOut(px.x,px.y,str);
	str.Format("%.0lf",right.x); dc.TextOut(px.x-10,px.y,str);
	if (fMenu==0)
	{
		px.x=m1*ipt[nPt-1].x+c1;
		px.y=m2*ipt[nPt-1].y+c2;
		rc=CRect(px.x,px.y,px.x+5,px.y+5);
		dc.Rectangle(rc);
	}
	if (fStatus)
	{
		for (i=0;i<=nPt-1;i++)
		{
			px.x=m1*ipt[i].x+c1;
			px.y=m2*ipt[i].y+c2;
			dc.Rectangle(px.x,px.y,px.x+5,px.y+5);
		}
		DrawCurve();
	}
}

void CCode7::DrawCurve()
{
	CClientDC dc(this);
	CString str;
	CPoint px,pxOld;

	// draw the curve
	CPen MyPen(PS_SOLID,2,RGB(100,100,100));
	dc.SelectObject(MyPen);
	dc.SelectObject(Arial80);
	for (int i=0;i<=m;i++)
	{
		px=CPoint((int)(m1*pt[i].x+c1),(int)(m2*pt[i].y+c2));
		if (i==0)
			dc.MoveTo(px);
		else
			if (curve.rc.PtInRect(px) && curve.rc.PtInRect(pxOld))
				dc.LineTo(px);
			else
				dc.MoveTo(px);
		pxOld=px;
	}
	if (fMenu==4)
	{
		Clear(output.rc);
		dc.TextOut(output.hm.x,output.hm.y,"Least-Square (Linear)");
		for (i=0;i<=1;i++)
		{
			str.Format("c[%d]=%lf",i,c[i]);
			dc.TextOut(output.hm.x+120,output.end.y-80+i*20,str);
		}
		dc.TextOut(output.hm.x+250,output.end.y-80,"y=c[0]+c[1] x");
	}
	if (fMenu==5)
	{
		Clear(output.rc);
		dc.TextOut(output.hm.x,output.hm.y,"Least-Square (Cubic)");
		for (i=0;i<=3;i++)
		{
			str.Format("c[%d]=%lf",i,c[i]);
			dc.TextOut(output.hm.x+120,output.end.y-80+i*20,str);
		}
		dc.TextOut(output.hm.x+250,output.end.y-80,
			"y=c[0]+c[1] x+c[2] x^2+c[3] x^3");
	}
}

void CCode7::Clear(CRect rc)
{
	CClientDC dc(this);
	CBrush whiteBrush(RGB(255,255,255));
	dc.FillRect(&rc,&whiteBrush);	
}

void CCode7::OnLButtonDown(UINT nFlags,CPoint px)
{
	CRect rc;
	for (int k=1;k<=nMenuItems;k++)
		if (menu[k].rc.PtInRect(px))
			if (nPt>1)
			{
				fMenu=k;			
				table.DestroyWindow();			
				switch(fMenu)
				{
					case 1: 
						Lagrange();  break;
					case 2:
						DDifference(); break;
					case 3: 
						CSpline(); break;
					case 4: 
					case 5: 
						LSquare();  break;
				}
				fStatus=1; 
				ShowTable(); 
				InvalidateRect(curve.rc);
			}
	if (curve.rc.PtInRect(px))
	{
		ipt[nPt].x=(px.x-c1)/m1;
		ipt[nPt].y=(px.y-c2)/m2;
		rc=CRect(px.x,px.y,px.x+5,px.y+5);
		InvalidateRect(rc);
		nPt++;
	}
}

void CCode7::ShowTable()
{
	CString str;
	CPoint hTable=CPoint(20,200);
	CRect rcTable=CRect(hTable.x,hTable.y,hTable.x+200,hTable.y+260);
	table.DestroyWindow();
    table.Create(WS_VISIBLE| WS_CHILD| WS_DLGFRAME| LVS_REPORT
        | LVS_NOSORTHEADER,rcTable,this,idc++);
	table.InsertColumn(0,"i",LVCFMT_CENTER,20);
	table.InsertColumn(1,"x[i]",LVCFMT_CENTER,45);
	table.InsertColumn(2,"y[i]",LVCFMT_CENTER,45);
	if (fMenu==3)
	{
		table.InsertColumn(3,"A[i]",LVCFMT_CENTER,45);
		table.InsertColumn(4,"B[i]",LVCFMT_CENTER,45);
		table.InsertColumn(5,"C[i]",LVCFMT_CENTER,45);
		table.InsertColumn(6,"D[i]",LVCFMT_CENTER,45);
	}
	for (int i=0;i<=nPt-1;i++)
	{
		str.Format("%d",i); table.InsertItem(i,str,0);
		str.Format("%.3lf",ipt[i].x); table.SetItemText(i,1,str);
		str.Format("%.3lf",ipt[i].y); table.SetItemText(i,2,str);
		if (fMenu==3 && i>0)
		{
			str.Format("%.3lf",A[i]); table.SetItemText(i,3,str);
			str.Format("%.3lf",B[i]); table.SetItemText(i,4,str);
			str.Format("%.3lf",C[i]); table.SetItemText(i,5,str);
			str.Format("%.3lf",D[i]); table.SetItemText(i,6,str);
		}
	}
}

void CCode7::Lagrange()
{
	Clear(output.rc);
	int i,j,k;
	double L, h=(ipt[nPt-1].x-ipt[0].x)/m;
	pt[0].x=ipt[0].x;
	pt[0].y=ipt[0].y;
	for (i=0;i<=m;i++)
	{
		pt[i].y=0;
		for (j=0;j<=nPt-1;j++)
		{
			L=1;
			for (k=0;k<=nPt-1;k++)
				if (j!=k)
					L *= (pt[i].x-ipt[k].x)/(ipt[j].x-ipt[k].x);
			pt[i].y += ipt[j].y*L;
		}
		if (i<m)
			pt[i+1].x=pt[i].x+h;
	}
}

void CCode7::DDifference()
{
	int i,j,k;
	double **d,product;
	double h=(ipt[nPt-1].x-ipt[0].x)/(double)m;
	d=new double *[maxPt+1];
	for (i=0;i<=maxPt+1;i++)
		d[i]=new double [maxPt+1];
	pt[0].x=ipt[0].x;
	for (i=0;i<=nPt-1;i++)
		d[0][i]=ipt[i].y;
	for (k=1;k<=nPt-1;k++)
		for (i=0;i<=nPt-1-k;i++)
			d[k][i]=(d[k-1][i+1]-d[k-1][i])/(ipt[k+i].x-ipt[i].x);
	for (i=0;i<=m;i++)
	{
		pt[i].y=d[0][0];
		for (j=1;j<=nPt-1;j++)
		{
			product=1;
			for (k=0;k<=j-1;k++)
				product *= (pt[i].x-ipt[k].x);
			pt[i].y += d[j][0]*product;
		}
		if (i<m)
			pt[i+1].x=pt[i].x+h;
	}
}

void CCode7::LSquare()
{
	int i,j,k,sa,sv;
	double **a,*b,*S,*v;
	double h=(ipt[nPt-1].x-ipt[0].x)/(double)m;
	pt[0].x=ipt[0].x;
	pt[0].y=ipt[0].y;
	max.y=pt[0].y; min.y=pt[0].y;
	switch (fMenu)
	{
		case 4: sa=2; sv=1; break;
		case 5: sa=6; sv=3; break;
	}
	a=new double *[sa+1];
	S=new double [sa+1];
	b=new double [sv+1];
	v=new double [sv+1];
	for (i=1;i<=sa+1;i++)
		a[i]=new double [sa+1];
	for (k=0;k<=sa;k++)
	{
		S[k]=0;
		if (k<=sv)
			v[k]=0;
	}
	for (k=0;k<=sa;k++)
		for (i=0;i<=nPt-1;i++)
		{
			S[k] += ((ipt[i].x==0&&k==0)?1:pow(ipt[i].x,k));
			if (k<=sv)
				v[k] += ipt[i].y*((ipt[i].x==0&&k==0)?1:pow(ipt[i].x,k));
		}
	
	//Form the SLE
	for (i=1;i<=sv+1;i++)        
	{
		for (j=1;j<=i;j++)
		{
			a[i][j]=S[i+j-2];
			a[j][i]=a[i][j];
		}
		b[i]=v[i-1];
	}

	// Solve the SLE
	double m1,Sum;
	for (k=1;k<=sv;k++)
		for (i=k+1;i<=sv+1;i++)
		{
			m1=a[i][k]/a[k][k];
			for (j=1;j<=sv+1;j++)
				a[i][j] -= m1*a[k][j];
			b[i] -= m1*b[k];
		}
	for (i=sv+1;i>=1;i--)
	{
    	Sum=0;
		c[i-1]=0;
		for (j=i;j<=sv+1;j++)
       		Sum += a[i][j]*c[j-1];
		c[i-1]=(b[i]-Sum)/a[i][i];
	}	
	for (i=0;i<=m;i++)
	{
		if (fMenu==4)
			pt[i].y=c[0]+c[1]*pt[i].x;
		if (fMenu==5)
			pt[i].y=c[0]+c[1]*pt[i].x+c[2]*pow(pt[i].x,2)+c[3]*pow(pt[i].x,3);
		if (i<m)
			pt[i+1].x=pt[i].x+h;
	}
}

void CCode7::CSpline()
{
	Clear(output.rc);
    int i,k;
	double *e,*f,*g,*r,*w;
	double h;
	e=new double [maxPt+1];
	f=new double [maxPt+1];
	g=new double [maxPt+1];
	r=new double [maxPt+1];
	w=new double [maxPt+1];
                                     
	// form the tridiagonal system
    e[0]=0; e[1]=0;
	f[0]=0; g[0]=0; r[0]=0;
	g[nPt-2]=0;
    for (k=1;k<=nPt-2;k++)
	{
        e[k+1]=ipt[k+1].x-ipt[k].x;
		f[k]=2*(ipt[k+1].x-ipt[k-1].x);
		g[k]=ipt[k+1].x-ipt[k].x;
		r[k]=6*((ipt[k+1].y-ipt[k].y)/(ipt[k+1].x-ipt[k].x)
			+(ipt[k-1].y-ipt[k].y)/(ipt[k].x-ipt[k-1].x));
	}

	// Thomas algorithm to solve the system
    w[0]=0; w[nPt-1]=0;
	for (k=2;k<=nPt-2;k++)
    {
		e[k]/=f[k-1];
		f[k]-=e[k]*g[k-1];
	}
    for (k=2;k<=nPt-2;k++)
    	r[k]-=e[k]*r[k-1];
    w[nPt-2]=r[nPt-2]/f[nPt-2];
    for (k=nPt-3;k>=1;k--)
    	w[k]=(r[k]-g[k]*w[k+1])/f[k];
    for (k=1;k<=nPt-1;k++)
    {
        A[k]=w[k-1]/(6*(ipt[k].x-ipt[k-1].x));
        B[k]=w[k]/(6*(ipt[k].x-ipt[k-1].x));
        C[k]=ipt[k-1].y/(ipt[k].x-ipt[k-1].x)
			-w[k-1]/6*(ipt[k].x-ipt[k-1].x);
        D[k]=ipt[k].y/(ipt[k].x-ipt[k-1].x)
			-w[k]/6*(ipt[k].x-ipt[k-1].x);
    }
	h=(ipt[nPt-1].x-ipt[0].x)/m;
	pt[0].x=ipt[0].x;
	pt[0].y=ipt[0].y;

	for (i=0;i<=m;i++)
	{
		for (k=1;k<=nPt-1;k++)
			if (pt[i].x>=ipt[k-1].x && pt[i].x<=ipt[k].x)
				pt[i].y=A[k]*pow(ipt[k].x-pt[i].x,3)+B[k]*pow(pt[i].x-ipt[k-1].x,3)
					+C[k]*(ipt[k].x-pt[i].x)+D[k]*(pt[i].x-ipt[k-1].x);
		if (i<m)
			pt[i+1].x=pt[i].x+h;
	}
	delete e,f,g,r,w;
}
